@@ -12,11 +12,9 @@
 #' ad <- read_umi_tools("...")
 #' }
 read_umi_tools <- function(filename, dtype = "float32") {
-  python_anndata <- reticulate::import("anndata")
-  ad <- AnnData()
-  ad$.__enclos_env__$private$.anndata <- python_anndata$read_umi_tools(
+  python_anndata <- reticulate::import("anndata", convert = FALSE)
+  py_to_r(python_anndata$read_umi_tools(
     filename = filename,
     dtype = dtype
-  )
-  ad
+  ))
 }
